function objectWithoutProperties (obj, exclude) { var target = {}; for (var k in obj) if (Object.prototype.hasOwnProperty.call(obj, k) && exclude.indexOf(k) === -1) target[k] = obj[k]; return target; }
import { writable } from 'svelte/store';
import { Router, stringify } from './vendor';

var cache = {};
var baseTag = document.getElementsByTagName('base');
var basePrefix = (baseTag[0] && baseTag[0].href) || '/';

export var ROOT_URL = basePrefix.replace(window.location.origin, '');

export var router = writable({
  path: '/',
  query: {},
  params: {},
  initial: true,
});

export var CTX_ROUTER = {};
export var CTX_ROUTE = {};

// use location.hash on embedded pages, e.g. Svelte REPL
export var HASHCHANGE = window.location.origin === 'null';

export function hashchangeEnable(value) {
  if (typeof value === 'boolean') {
    HASHCHANGE = !!value;
  }

  return HASHCHANGE;
}

export function fixedLocation(path, callback, doFinally) {
  var baseUri = HASHCHANGE ? window.location.hash.replace('#', '') : window.location.pathname;

  // this will rebase anchors to avoid location changes
  if (path.charAt() !== '/') {
    path = baseUri + path;
  }

  var currentURL = baseUri + window.location.hash + window.location.search;

  // do not change location et all...
  if (currentURL !== path) {
    callback(path);
  }

  // invoke final guard regardless of previous result
  if (typeof doFinally === 'function') {
    doFinally();
  }
}

export function cleanPath(uri, fix) {
  return uri !== '/' || fix ? uri.replace(/\/$/, '') : uri;
}

export function navigateTo(path, options) {
  var ref = options || {};
  var reload = ref.reload;
  var replace = ref.replace;
  var params = ref.params;
  var queryParams = ref.queryParams;

  // If path empty or no string, throws error
  if (!path || typeof path !== 'string' || (path[0] !== '/' && path[0] !== '#')) {
    throw new Error(("Expecting '/" + path + "' or '#" + path + "', given '" + path + "'"));
  }

  if (params) {
    path = path.replace(/:([a-zA-Z][a-zA-Z0-9_-]*)/g, function (_, key) { return params[key]; });
  }

  if (queryParams) {
    var qs = stringify(queryParams);

    if (qs) {
      path += "?" + qs;
    }
  }

  if (HASHCHANGE) {
    var fixedURL = path.replace(/^#|#$/g, '');

    if (ROOT_URL !== '/') {
      fixedURL = fixedURL.replace(cleanPath(ROOT_URL), '');
    }

    window.location.hash = fixedURL !== '/' ? fixedURL : '';
    return;
  }

  // If no History API support, fallbacks to URL redirect
  if (reload || !window.history.pushState || !window.dispatchEvent) {
    window.location.href = path;
    return;
  }

  // If has History API support, uses it
  fixedLocation(path, function (nextURL) {
    window.history[replace ? 'replaceState' : 'pushState'](null, '', nextURL);
    window.dispatchEvent(new Event('popstate'));
  });
}

export function getProps(given, required) {
  var sub = given.props;
  var rest = objectWithoutProperties( given, ["props"] );
  var others = rest;

  // prune all declared props from this component
  required.forEach(function (k) {
    delete others[k];
  });

  return Object.assign({}, sub,
    others);
}

export function isActive(uri, path, exact) {
  if (!cache[[uri, path, exact]]) {
    if (exact !== true && path.indexOf(uri) === 0) {
      cache[[uri, path, exact]] = /^[#/?]?$/.test(path.substr(uri.length, 1));
    } else if (uri.includes('*') || uri.includes(':')) {
      cache[[uri, path, exact]] = Router.matches(uri, path);
    } else {
      cache[[uri, path, exact]] = cleanPath(path) === uri;
    }
  }

  return cache[[uri, path, exact]];
}

export function isPromise(object) {
  return object && typeof object.then === 'function';
}

export function isSvelteComponent(object) {
  return object && object.prototype;
}
